@@ -25,6 +25,7 @@ import (
 func (f File) Validate(c path.ContextPath) (r report.Report) {
 	r.Merge(f.Node.Validate(c))
 	r.AddOnError(c.Append("mode"), validateMode(f.Mode))
+	r.AddOnWarn(c.Append("mode"), validateModeSpecialBits(f.Mode))
 	r.AddOnError(c.Append("overwrite"), f.validateOverwrite())
 	return
 }
@@ -41,3 +42,28 @@ func (f FileEmbedded1) IgnoreDuplicates() map[string]struct{} {
 		"Append": {},
 	}
 }
+
+func (fc FileContents) Validate(c path.ContextPath) (r report.Report) {
+	r.AddOnError(c.Append("compression"), fc.validateCompression())
+	r.AddOnError(c.Append("verification", "hash"), fc.validateVerification())
+	r.AddOnError(c.Append("source"), validateURLNilOK(fc.Source))
+	return
+}
+
+func (fc FileContents) validateCompression() error {
+	if fc.Compression != nil {
+		switch *fc.Compression {
+		case "", "gzip":
+		default:
+			return errors.ErrCompressionInvalid
+		}
+	}
+	return nil
+}
+
+func (fc FileContents) validateVerification() error {
+	if fc.Verification.Hash != nil && fc.Source == nil {
+		return errors.ErrVerificationAndNilSource
+	}
+	return nil
+}
