@@ -23,17 +23,22 @@ import (
 	"github.com/coreos/vcontext/report"
 )
 
-func (v Ignition) Semver() (*semver.Version, error) {
-	return semver.NewVersion(v.Version)
+func (c ConfigReference) Key() string {
+	if c.Source == nil {
+		return ""
+	}
+	return *c.Source
 }
 
-func (ic IgnitionConfig) Validate(c path.ContextPath) (r report.Report) {
-	for i, res := range ic.Merge {
-		r.AddOnError(c.Append("merge", i), res.validateRequiredSource())
-	}
+func (cr ConfigReference) Validate(c path.ContextPath) (r report.Report) {
+	r.AddOnError(c.Append("source"), validateURLNilOK(cr.Source))
 	return
 }
 
+func (v Ignition) Semver() (*semver.Version, error) {
+	return semver.NewVersion(v.Version)
+}
+
 func (v Ignition) Validate(c path.ContextPath) (r report.Report) {
 	c = c.Append("version")
 	tv, err := v.Semver()
