@@ -32,3 +32,7 @@ import (
 func FetchConfig(_ *resource.Fetcher) (types.Config, report.Report, error) {
 	return types.Config{}, report.Report{}, errors.New("vmware provider is not supported on this architecture")
 }
+
+func DelConfig(_ *resource.Fetcher) error {
+	return errors.New("vmware provider is not supported on this architecture")
+}
