@@ -39,6 +39,7 @@ import (
 	"google.golang.org/api/option"
 
 	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/arn"
 	"github.com/aws/aws-sdk-go/aws/awserr"
 	"github.com/aws/aws-sdk-go/aws/credentials"
 	"github.com/aws/aws-sdk-go/aws/session"
@@ -62,6 +63,15 @@ var (
 		"Accept-Encoding": []string{"identity"},
 		"Accept":          []string{"application/vnd.coreos.ignition+json;version=3.3.0, */*;q=0.1"},
 	}
+
+	// We could derive this info from aws-sdk-go/aws/endpoints/defaults.go,
+	// but hardcoding it allows us to unit-test that specific regions
+	// are used for hinting
+	awsPartitionRegionHints = map[string]string{
+		"aws":        "us-east-1",
+		"aws-cn":     "cn-north-1",
+		"aws-us-gov": "us-gov-west-1",
+	}
 )
 
 // Fetcher holds settings for fetching resources from URLs
@@ -135,7 +145,7 @@ func (f *Fetcher) FetchToBuffer(u url.URL, opts FetchOptions) ([]byte, error) {
 		err = f.fetchFromTFTP(u, dest, opts)
 	case "data":
 		err = f.fetchFromDataURL(u, dest, opts)
-	case "s3":
+	case "s3", "arn":
 		buf := &s3buf{
 			WriteAtBuffer: aws.NewWriteAtBuffer([]byte{}),
 		}
@@ -196,7 +206,7 @@ func (f *Fetcher) Fetch(u url.URL, dest *os.File, opts FetchOptions) error {
 		return f.fetchFromTFTP(u, dest, opts)
 	case "data":
 		return f.fetchFromDataURL(u, dest, opts)
-	case "s3":
+	case "s3", "arn":
 		return f.fetchFromS3(u, dest, opts)
 	case "gs":
 		return f.fetchFromGCS(u, dest, opts)
@@ -344,18 +354,23 @@ func (f *Fetcher) fetchFromDataURL(u url.URL, dest io.Writer, opts FetchOptions)
 // credentials to fetch the object content.
 func (f *Fetcher) fetchFromGCS(u url.URL, dest io.Writer, opts FetchOptions) error {
 	ctx := context.Background()
-	var clientOption option.ClientOption
 	if f.GCSSession == nil {
+		clientOption := option.WithoutAuthentication()
 		if metadata.OnGCE() {
-			id, _ := metadata.ProjectID()
-			creds := &google.Credentials{
-				ProjectID:   id,
-				TokenSource: google.ComputeTokenSource("", storage.ScopeReadOnly),
+			// check whether the VM is associated with a service
+			// account
+			if _, err := metadata.Scopes(""); err == nil {
+				id, _ := metadata.ProjectID()
+				creds := &google.Credentials{
+					ProjectID:   id,
+					TokenSource: google.ComputeTokenSource("", storage.ScopeReadOnly),
+				}
+				clientOption = option.WithCredentials(creds)
+			} else {
+				f.Logger.Debug("falling back to unauthenticated GCS access: %v", err)
 			}
-			clientOption = option.WithCredentials(creds)
 		} else {
-			f.Logger.Debug("falling back to unauthenticated GCS access")
-			clientOption = option.WithoutAuthentication()
+			f.Logger.Debug("falling back to unauthenticated GCS access: not running in GCE")
 		}
 
 		var err error
@@ -407,17 +422,50 @@ func (f *Fetcher) fetchFromS3(u url.URL, dest s3target, opts FetchOptions) error
 	}
 	sess := f.AWSSession.Copy()
 
+	// Determine the bucket and key based on the URL scheme
+	var bucket, key, region, regionHint string
+	var err error
+	switch u.Scheme {
+	case "s3":
+		bucket = u.Host
+		key = u.Path
+	case "arn":
+		fullURL := u.Scheme + ":" + u.Opaque
+		// Parse the bucket and key from the ARN Resource.
+		// Also set the region for accesspoints.
+		// S3 bucket ARNs don't include the region field.
+		bucket, key, region, regionHint, err = f.parseARN(fullURL)
+		if err != nil {
+			return err
+		}
+	default:
+		return ErrSchemeUnsupported
+	}
+
 	// Determine the partition and region this bucket is in
-	regionHint := "us-east-1"
-	if f.S3RegionHint != "" {
-		regionHint = f.S3RegionHint
-	}
-	region, err := s3manager.GetBucketRegion(ctx, sess, u.Host, regionHint)
-	if err != nil {
-		if aerr, ok := err.(awserr.Error); ok && aerr.Code() == "NotFound" {
-			return fmt.Errorf("couldn't determine the region for bucket %q: %v", u.Host, err)
+	if region == "" {
+		// We didn't get an accesspoint ARN, so we don't know the
+		// region directly.  Maybe we computed a region hint from
+		// the ARN partition field?
+		if regionHint == "" {
+			// Nope; we got an unknown ARN partition value or an
+			// s3:// URL.  Maybe we're running in AWS and can
+			// assume the same partition we're running in?
+			regionHint = f.S3RegionHint
+		}
+		if regionHint == "" {
+			// Nope; assume aws partition.
+			regionHint = "us-east-1"
+		}
+		// Use the region hint to ask the correct partition for the
+		// bucket's region.
+		region, err = s3manager.GetBucketRegion(ctx, sess, bucket, regionHint)
+		if err != nil {
+			if aerr, ok := err.(awserr.Error); ok && aerr.Code() == "NotFound" {
+				return fmt.Errorf("couldn't determine the region for bucket %q: %v", bucket, err)
+			}
+			return err
 		}
-		return err
 	}
 
 	sess.Config.Region = aws.String(region)
@@ -428,8 +476,8 @@ func (f *Fetcher) fetchFromS3(u url.URL, dest s3target, opts FetchOptions) error
 	}
 
 	input := &s3.GetObjectInput{
-		Bucket:    &u.Host,
-		Key:       &u.Path,
+		Bucket:    &bucket,
+		Key:       &key,
 		VersionId: versionId,
 	}
 	err = f.fetchFromS3WithCreds(ctx, dest, input, sess)
@@ -464,7 +512,7 @@ func (f *Fetcher) fetchFromS3WithCreds(ctx context.Context, dest s3target, input
 		return err
 	}
 
-	awsConfig := aws.NewConfig().WithHTTPClient(httpClient)
+	awsConfig := aws.NewConfig().WithHTTPClient(httpClient).WithUseDualStack(true)
 	s3Client := s3.New(sess, awsConfig)
 	downloader := s3manager.NewDownloaderWithClient(s3Client)
 	if _, err := downloader.DownloadWithContext(ctx, dest, input); err != nil {
@@ -522,3 +570,56 @@ func (f *Fetcher) decompressCopyHashAndVerify(dest io.Writer, src io.Reader, opt
 	}
 	return nil
 }
+
+// parseARN is a custom wrapper around arn.Parse(); it takes arnURL, a full ARN URL,
+// and returns a bucket, a key, a potentially empty region, and a
+// potentially empty region hint for use in region detection; or an error if
+// the ARN is invalid or not for an S3 object.
+// If the given arnURL is an accesspoint ARN, the region is set.
+// The region is empty for S3 bucket ARNs because they don't include the region field.
+func (f *Fetcher) parseARN(arnURL string) (string, string, string, string, error) {
+	if !arn.IsARN(arnURL) {
+		return "", "", "", "", configErrors.ErrInvalidS3ARN
+	}
+	s3arn, err := arn.Parse(arnURL)
+	if err != nil {
+		return "", "", "", "", err
+	}
+	if s3arn.Service != "s3" {
+		return "", "", "", "", configErrors.ErrInvalidS3ARN
+	}
+	// empty if unrecognized partition
+	regionHint := awsPartitionRegionHints[s3arn.Partition]
+	// Split the ARN bucket (or accesspoint) and key by separating on slashes.
+	// See https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arns-paths for more info.
+	urlSplit := strings.Split(arnURL, "/")
+
+	// Determine if the ARN is for an access point or a bucket.
+	if strings.HasPrefix(s3arn.Resource, "accesspoint/") {
+		// urlSplit must consist of arn, name of accesspoint, "object",
+		// and key
+		if len(urlSplit) < 4 || urlSplit[2] != "object" {
+			return "", "", "", "", configErrors.ErrInvalidS3ARN
+		}
+
+		// When using GetObjectInput with an access point,
+		// you provide the access point ARN in place of the bucket name.
+		// For more information about access point ARNs, see Using access points
+		// https://docs.aws.amazon.com/AmazonS3/latest/userguide/using-access-points.html
+		bucket := strings.Join(urlSplit[:2], "/")
+		key := strings.Join(urlSplit[3:], "/")
+		return bucket, key, s3arn.Region, regionHint, nil
+	}
+	// urlSplit must consist of name of bucket and key
+	if len(urlSplit) < 2 {
+		return "", "", "", "", configErrors.ErrInvalidS3ARN
+	}
+
+	// Parse out the bucket name in order to find the region with s3manager.GetBucketRegion.
+	// If specified, the key is part of the Relative ID which has the format "bucket-name/object-key" according to
+	// https://docs.aws.amazon.com/AmazonS3/latest/userguide/s3-arn-format.html
+	bucketUrlSplit := strings.Split(urlSplit[0], ":")
+	bucket := bucketUrlSplit[len(bucketUrlSplit)-1]
+	key := strings.Join(urlSplit[1:], "/")
+	return bucket, key, "", regionHint, nil
+}
